@@ -1,9 +1,9 @@
 import html
 from typing import List, Optional
 
-import lxml
 from lxml import etree
 
+from reqif.helpers.lxml import stringify_namespaced_children
 from reqif.models.reqif_spec_object import (
     ReqIFSpecObject,
     SpecObjectAttribute,
@@ -26,6 +26,14 @@ ATTRIBUTE_INTEGER_TEMPLATE = """\
             </ATTRIBUTE-VALUE-INTEGER>
 """
 
+ATTRIBUTE_REAL_TEMPLATE = """\
+            <ATTRIBUTE-VALUE-REAL THE-VALUE="{value}">
+              <DEFINITION>
+                <ATTRIBUTE-DEFINITION-REAL-REF>{definition_ref}</ATTRIBUTE-DEFINITION-REAL-REF>
+              </DEFINITION>
+            </ATTRIBUTE-VALUE-REAL>
+"""
+
 ATTRIBUTE_ENUMERATION_TEMPLATE = """\
             <ATTRIBUTE-VALUE-ENUMERATION>
               <VALUES>
@@ -47,14 +55,20 @@ ATTRIBUTE_ENUMERATION_TEMPLATE_REVERSE = """\
             </ATTRIBUTE-VALUE-ENUMERATION>
 """
 
+ATTRIBUTE_DATE_TEMPLATE = """\
+            <ATTRIBUTE-VALUE-DATE THE-VALUE="{value}">
+              <DEFINITION>
+                <ATTRIBUTE-DEFINITION-DATE-REF>{definition_ref}</ATTRIBUTE-DEFINITION-DATE-REF>
+              </DEFINITION>
+            </ATTRIBUTE-VALUE-DATE>
+"""
+
 ATTRIBUTE_XHTML_TEMPLATE = """\
             <ATTRIBUTE-VALUE-XHTML>
               <DEFINITION>
                 <ATTRIBUTE-DEFINITION-XHTML-REF>{definition_ref}</ATTRIBUTE-DEFINITION-XHTML-REF>
               </DEFINITION>
-              <THE-VALUE>
-{value}
-              </THE-VALUE>
+              <THE-VALUE>{value}</THE-VALUE>
             </ATTRIBUTE-VALUE-XHTML>
 """
 
@@ -128,6 +142,20 @@ class SpecObjectParser:
                     definition_ref=attribute_definition_ref,
                     value=attribute_value,
                 )
+            elif attribute_xml.tag == "ATTRIBUTE-VALUE-REAL":
+                attribute_value = attribute_xml.attrib["THE-VALUE"]
+
+                attribute_definition_ref = (
+                    attribute_xml.find("DEFINITION")
+                    .find("ATTRIBUTE-DEFINITION-REAL-REF")
+                    .text
+                )
+                attribute = SpecObjectAttribute(
+                    xml_node=attribute_xml,
+                    attribute_type=SpecObjectAttributeType.REAL,
+                    definition_ref=attribute_definition_ref,
+                    value=attribute_value,
+                )
             elif attribute_xml.tag == "ATTRIBUTE-VALUE-BOOLEAN":
                 attribute_value = attribute_xml.attrib["THE-VALUE"]
 
@@ -142,21 +170,23 @@ class SpecObjectParser:
                     definition_ref=attribute_definition_ref,
                     value=attribute_value,
                 )
+            elif attribute_xml.tag == "ATTRIBUTE-VALUE-DATE":
+                attribute_value = attribute_xml.attrib["THE-VALUE"]
+
+                attribute_definition_ref = (
+                    attribute_xml.find("DEFINITION")
+                    .find("ATTRIBUTE-DEFINITION-DATE-REF")
+                    .text
+                )
+                attribute = SpecObjectAttribute(
+                    xml_node=attribute_xml,
+                    attribute_type=SpecObjectAttributeType.DATE,
+                    definition_ref=attribute_definition_ref,
+                    value=attribute_value,
+                )
             elif attribute_xml.tag == "ATTRIBUTE-VALUE-XHTML":
                 the_value = attribute_xml.find("THE-VALUE")
-                # TODO: This does not work:
-                # <THE-VALUE xmlns:xhtml="http://www.w3.org/1999/xhtml">
-                # is printed.
-                # the_value.tag = etree.QName(the_value).localname
-                # etree.cleanup_namespaces(the_value)
-                attribute_value_decoded_lines = (
-                    lxml.etree.tostring(the_value, method="xml")
-                    .decode("utf8")
-                    .rstrip()
-                )
-                attribute_value = "\n".join(
-                    attribute_value_decoded_lines.split("\n")[1:-1]
-                )
+                attribute_value = stringify_namespaced_children(the_value)
                 attribute_definition_ref = (
                     attribute_xml.find("DEFINITION")
                     .find("ATTRIBUTE-DEFINITION-XHTML-REF")
@@ -238,17 +268,21 @@ class SpecObjectParser:
         output = ""
         output += "          <VALUES>\n"
         for attribute in spec_object.attributes:
-            attribute_value = html.escape(attribute.value)
             if attribute.attribute_type == SpecObjectAttributeType.STRING:
                 output += ATTRIBUTE_STRING_TEMPLATE.format(
                     definition_ref=attribute.definition_ref,
-                    value=attribute_value,
+                    value=html.escape(attribute.value),
                 )
             elif attribute.attribute_type == SpecObjectAttributeType.INTEGER:
                 output += ATTRIBUTE_INTEGER_TEMPLATE.format(
                     definition_ref=attribute.definition_ref,
                     value=attribute.value,
                 )
+            elif attribute.attribute_type == SpecObjectAttributeType.REAL:
+                output += ATTRIBUTE_REAL_TEMPLATE.format(
+                    definition_ref=attribute.definition_ref,
+                    value=attribute.value,
+                )
             elif (
                 attribute.attribute_type == SpecObjectAttributeType.ENUMERATION
             ):
@@ -272,11 +306,18 @@ class SpecObjectParser:
                         definition_ref=attribute.definition_ref,
                         value=attribute.value,
                     )
+            elif attribute.attribute_type == SpecObjectAttributeType.DATE:
+                output += ATTRIBUTE_DATE_TEMPLATE.format(
+                    definition_ref=attribute.definition_ref,
+                    value=attribute.value,
+                )
             elif attribute.attribute_type == SpecObjectAttributeType.XHTML:
                 output += ATTRIBUTE_XHTML_TEMPLATE.format(
                     definition_ref=attribute.definition_ref,
                     value=attribute.value,
                 )
+            else:
+                raise NotImplementedError(attribute)
 
         output += "          </VALUES>\n"
         return output
