@@ -1,7 +1,6 @@
 from typing import List, Optional
 
-import lxml
-
+from reqif.helpers.lxml import stringify_namespaced_children
 from reqif.models.reqif_spec_object import SpecObjectAttribute
 from reqif.models.reqif_specification import (
     ReqIFSpecification,
@@ -10,12 +9,15 @@ from reqif.models.reqif_types import SpecObjectAttributeType
 from reqif.parsers.spec_hierarchy_parser import (
     ReqIFSpecHierarchyParser,
 )
-from reqif.parsers.spec_object_parser import ATTRIBUTE_XHTML_TEMPLATE
+from reqif.parsers.spec_object_parser import (
+    ATTRIBUTE_XHTML_TEMPLATE,
+    ATTRIBUTE_DATE_TEMPLATE,
+)
 
 
 class ReqIFSpecificationParser:
     @staticmethod
-    def parse(specification_xml):
+    def parse(specification_xml) -> ReqIFSpecification:
         assert "SPECIFICATION" in specification_xml.tag, f"{specification_xml}"
 
         attributes = specification_xml.attrib
@@ -57,7 +59,7 @@ class ReqIFSpecificationParser:
                 children_xml = specification_child_xml
 
         children = None
-        if children_xml is not None and len(children_xml):
+        if children_xml is not None:
             children = []
             for child_xml in children_xml:
                 spec_hierarchy_xml = ReqIFSpecHierarchyParser.parse(child_xml)
@@ -67,8 +69,8 @@ class ReqIFSpecificationParser:
         xml_values = specification_xml.find("VALUES")
         if xml_values is not None:
             values = []
-            if len(xml_values) > 0:
-                xml_attribute = xml_values[0]
+            for xml_value in xml_values:
+                xml_attribute = xml_value
                 if xml_attribute.tag == "ATTRIBUTE-VALUE-STRING":
                     attribute_value = xml_attribute.attrib["THE-VALUE"]
                     definition_ref = xml_attribute[0][0].text
@@ -79,21 +81,19 @@ class ReqIFSpecificationParser:
                         value=attribute_value,
                     )
                     values.append(values_attribute)
+                elif xml_attribute.tag == "ATTRIBUTE-VALUE-DATE":
+                    attribute_value = xml_attribute.attrib["THE-VALUE"]
+                    definition_ref = xml_attribute[0][0].text
+                    values_attribute = SpecObjectAttribute(
+                        xml_node=xml_attribute,
+                        attribute_type=SpecObjectAttributeType.DATE,
+                        definition_ref=definition_ref,
+                        value=attribute_value,
+                    )
+                    values.append(values_attribute)
                 elif xml_attribute.tag == "ATTRIBUTE-VALUE-XHTML":
                     the_value = xml_attribute.find("THE-VALUE")
-                    # TODO: This does not work:
-                    # <THE-VALUE xmlns:xhtml="http://www.w3.org/1999/xhtml">
-                    # is printed.
-                    # the_value.tag = etree.QName(the_value).localname
-                    # etree.cleanup_namespaces(the_value)
-                    attribute_value_decoded_lines = (
-                        lxml.etree.tostring(the_value, method="xml")
-                        .decode("utf8")
-                        .rstrip()
-                    )
-                    attribute_value = "\n".join(
-                        attribute_value_decoded_lines.split("\n")[1:-1]
-                    )
+                    attribute_value = stringify_namespaced_children(the_value)
                     attribute_name = (
                         xml_attribute.find("DEFINITION")
                         .find("ATTRIBUTE-DEFINITION-XHTML-REF")
@@ -106,7 +106,8 @@ class ReqIFSpecificationParser:
                         value=attribute_value,
                     )
                     values.append(values_attribute)
-
+                else:
+                    raise NotImplementedError(xml_attribute)
         return ReqIFSpecification(
             xml_node=specification_xml,
             description=description,
@@ -168,6 +169,14 @@ class ReqIFSpecificationParser:
                         output += "          <VALUES>\n"
                         for xml_attribute in xml_values_attributes:
                             if (
+                                xml_attribute.attribute_type
+                                == SpecObjectAttributeType.DATE
+                            ):
+                                output += ATTRIBUTE_DATE_TEMPLATE.format(
+                                    definition_ref=xml_attribute.definition_ref,
+                                    value=xml_attribute.value,
+                                )
+                            elif (
                                 xml_attribute.attribute_type
                                 == SpecObjectAttributeType.XHTML
                             ):
@@ -176,7 +185,7 @@ class ReqIFSpecificationParser:
                                     value=xml_attribute.value,
                                 )
                             else:
-                                raise NotImplementedError
+                                raise NotImplementedError(xml_attribute)
                         output += "          </VALUES>\n"
         output += "        </SPECIFICATION>\n"
 
